@@ -3,7 +3,7 @@
 //! Insertion and popping the largest element have *O*(log(*n*)) time complexity.
 //! Checking the largest element is *O*(1). Converting a vector to a *d*-ary heap
 //! can be done in-place, and has *O*(*n*) complexity. A *d*-ary heap can also be
-//! converted to a sorted vector in-place, allowing it to be used for an *O*(*n* \* log(*n*))
+//! converted to a sorted vector in-place, allowing it to be used for an *O*(*n* * log(*n*))
 //! in-place heapsort.
 //!
 //! # Comparison to standard library
@@ -249,6 +249,8 @@ use core::ops::{Deref, DerefMut};
 use core::ptr;
 use core::slice;
 
+#[cfg(feature = "unstable")]
+use alloc::collections::TryReserveError;
 use alloc::{vec, vec::Vec};
 
 /// A binary heap (*d* = 2).
@@ -277,12 +279,12 @@ pub type OctonaryHeap<T> = DaryHeap<T, 8>;
 /// This will be a max-heap.
 ///
 /// It is a logic error for an item to be modified in such a way that the
-/// item's ordering relative to any other item, as determined by the `Ord`
+/// item's ordering relative to any other item, as determined by the [`Ord`]
 /// trait, changes while it is in the heap. This is normally only possible
-/// through `Cell`, `RefCell`, global state, I/O, or unsafe code. The
-/// behavior resulting from such a logic error is not specified, but will
-/// not result in undefined behavior. This could include panics, incorrect
-/// results, aborts, memory leaks, and non-termination.
+/// through [`Cell`], [`RefCell`], global state, I/O, or unsafe code. The
+/// behavior resulting from such a logic error is not specified (it
+/// could include panics, incorrect results, aborts, memory leaks, or
+/// non-termination) but will not be undefined behavior.
 ///
 /// # Usage
 ///
@@ -355,7 +357,7 @@ pub type OctonaryHeap<T> = DaryHeap<T, 8>;
 ///
 /// ## Min-heap
 ///
-/// Either `std::cmp::Reverse` or a custom `Ord` implementation can be used to
+/// Either [`core::cmp::Reverse`] or a custom [`Ord`] implementation can be used to
 /// make `DaryHeap` a min-heap. This makes `heap.pop()` return the smallest
 /// value instead of the greatest one.
 ///
@@ -379,13 +381,17 @@ pub type OctonaryHeap<T> = DaryHeap<T, 8>;
 ///
 /// # Time complexity
 ///
-/// | [push] | [pop]     | [peek]/[peek\_mut] |
-/// |--------|-----------|--------------------|
-/// | O(1)~  | *O*(log(*n*)) | *O*(1)               |
+/// | [push]  | [pop]         | [peek]/[peek\_mut] |
+/// |---------|---------------|--------------------|
+/// | *O*(1)~ | *O*(log(*n*)) | *O*(1)             |
 ///
 /// The value for `push` is an expected cost; the method documentation gives a
 /// more detailed analysis.
 ///
+/// [`core::cmp::Reverse`]: core::cmp::Reverse
+/// [`Ord`]: core::cmp::Ord
+/// [`Cell`]: core::cell::Cell
+/// [`RefCell`]: core::cell::RefCell
 /// [push]: DaryHeap::push
 /// [pop]: DaryHeap::pop
 /// [peek]: DaryHeap::peek
@@ -520,6 +526,7 @@ impl<T: Ord, const D: usize> DaryHeap<T, D> {
     /// let mut heap = QuaternaryHeap::new();
     /// heap.push(4);
     /// ```
+    #[must_use]
     pub fn new() -> DaryHeap<T, D> {
         DaryHeap { data: vec![] }
     }
@@ -538,6 +545,7 @@ impl<T: Ord, const D: usize> DaryHeap<T, D> {
     /// let mut heap = QuaternaryHeap::with_capacity(10);
     /// heap.push(4);
     /// ```
+    #[must_use]
     pub fn with_capacity(capacity: usize) -> DaryHeap<T, D> {
         DaryHeap {
             data: Vec::with_capacity(capacity),
@@ -671,6 +679,7 @@ impl<T: Ord, const D: usize> DaryHeap<T, D> {
     /// let vec = heap.into_sorted_vec();
     /// assert_eq!(vec, [1, 2, 3, 4, 5, 6, 7]);
     /// ```
+    #[must_use = "`self` will be dropped if the result is not used"]
     pub fn into_sorted_vec(mut self) -> Vec<T> {
         let mut end = self.len();
         while end > 1 {
@@ -1040,6 +1049,7 @@ impl<T, const D: usize> DaryHeap<T, D> {
     /// # Time complexity
     ///
     /// Cost is *O*(1) in the worst case.
+    #[must_use]
     pub fn peek(&self) -> Option<&T> {
         self.data.get(0)
     }
@@ -1056,6 +1066,7 @@ impl<T, const D: usize> DaryHeap<T, D> {
     /// assert!(heap.capacity() >= 100);
     /// heap.push(4);
     /// ```
+    #[must_use]
     pub fn capacity(&self) -> usize {
         self.data.capacity()
     }
@@ -1110,6 +1121,84 @@ impl<T, const D: usize> DaryHeap<T, D> {
         self.data.reserve(additional);
     }
 
+    /// Tries to reserve the minimum capacity for exactly `additional`
+    /// elements to be inserted in the given `DaryHeap`. After calling
+    /// `try_reserve_exact`, capacity will be greater than or equal to
+    /// `self.len() + additional` if it returns `Ok(())`.
+    /// Does nothing if the capacity is already sufficient.
+    ///
+    /// Note that the allocator may give the collection more space than it
+    /// requests. Therefore, capacity can not be relied upon to be precisely
+    /// minimal. Prefer [`try_reserve`] if future insertions are expected.
+    ///
+    /// [`try_reserve`]: DaryHeap::try_reserve
+    ///
+    /// # Errors
+    ///
+    /// If the capacity overflows, or the allocator reports a failure, then an error
+    /// is returned.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use dary_heap::BinaryHeap;
+    /// use std::collections::TryReserveError;
+    ///
+    /// fn find_max_slow(data: &[u32]) -> Result<Option<u32>, TryReserveError> {
+    ///     let mut heap = BinaryHeap::new();
+    ///
+    ///     // Pre-reserve the memory, exiting if we can't
+    ///     heap.try_reserve_exact(data.len())?;
+    ///
+    ///     // Now we know this can't OOM in the middle of our complex work
+    ///     heap.extend(data.iter());
+    ///
+    ///     Ok(heap.pop())
+    /// }
+    /// # find_max_slow(&[1, 2, 3]).expect("why is the test harness OOMing on 12 bytes?");
+    /// ```
+    #[cfg(feature = "unstable")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "unstable")))]
+    pub fn try_reserve_exact(&mut self, additional: usize) -> Result<(), TryReserveError> {
+        self.data.try_reserve_exact(additional)
+    }
+
+    /// Tries to reserve capacity for at least `additional` more elements to be inserted
+    /// in the given `DaryHeap`. The collection may reserve more space to avoid
+    /// frequent reallocations. After calling `try_reserve`, capacity will be
+    /// greater than or equal to `self.len() + additional`. Does nothing if
+    /// capacity is already sufficient.
+    ///
+    /// # Errors
+    ///
+    /// If the capacity overflows, or the allocator reports a failure, then an error
+    /// is returned.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use dary_heap::QuaternaryHeap;
+    /// use std::collections::TryReserveError;
+    ///
+    /// fn find_max_slow(data: &[u32]) -> Result<Option<u32>, TryReserveError> {
+    ///     let mut heap = QuaternaryHeap::new();
+    ///
+    ///     // Pre-reserve the memory, exiting if we can't
+    ///     heap.try_reserve(data.len())?;
+    ///
+    ///     // Now we know this can't OOM in the middle of our complex work
+    ///     heap.extend(data.iter());
+    ///
+    ///     Ok(heap.pop())
+    /// }
+    /// # find_max_slow(&[1, 2, 3]).expect("why is the test harness OOMing on 12 bytes?");
+    /// ```
+    #[cfg(feature = "unstable")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "unstable")))]
+    pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError> {
+        self.data.try_reserve(additional)
+    }
+
     /// Discards as much additional capacity as possible.
     ///
     /// # Examples
@@ -1169,6 +1258,7 @@ impl<T, const D: usize> DaryHeap<T, D> {
     /// ```
     #[cfg(feature = "unstable")]
     #[cfg_attr(docsrs, doc(cfg(feature = "unstable")))]
+    #[must_use]
     pub fn as_slice(&self) -> &[T] {
         self.data.as_slice()
     }
@@ -1190,6 +1280,7 @@ impl<T, const D: usize> DaryHeap<T, D> {
     ///     println!("{}", x);
     /// }
     /// ```
+    #[must_use = "`self` will be dropped if the result is not used"]
     pub fn into_vec(self) -> Vec<T> {
         self.into()
     }
@@ -1206,6 +1297,7 @@ impl<T, const D: usize> DaryHeap<T, D> {
     ///
     /// assert_eq!(heap.len(), 2);
     /// ```
+    #[must_use]
     pub fn len(&self) -> usize {
         self.data.len()
     }
@@ -1228,6 +1320,7 @@ impl<T, const D: usize> DaryHeap<T, D> {
     ///
     /// assert!(!heap.is_empty());
     /// ```
+    #[must_use]
     pub fn is_empty(&self) -> bool {
         self.len() == 0
     }
@@ -1432,6 +1525,7 @@ impl<T> Drop for Hole<'_, T> {
 /// documentation for more.
 ///
 /// [`iter`]: DaryHeap::iter
+#[must_use = "iterators are lazy and do nothing unless consumed"]
 pub struct Iter<'a, T: 'a> {
     iter: slice::Iter<'a, T>,
 }
@@ -1489,9 +1583,10 @@ impl<T> FusedIterator for Iter<'_, T> {}
 /// An owning iterator over the elements of a `DaryHeap`.
 ///
 /// This `struct` is created by [`DaryHeap::into_iter()`]
-/// (provided by the `IntoIterator` trait). See its documentation for more.
+/// (provided by the [`IntoIterator`] trait). See its documentation for more.
 ///
 /// [`into_iter`]: DaryHeap::into_iter
+/// [`IntoIterator`]: core::iter::IntoIterator
 #[derive(Clone)]
 pub struct IntoIter<T> {
     iter: vec::IntoIter<T>,
@@ -1550,6 +1645,7 @@ unsafe impl<T> core::iter::SourceIter for IntoIter<T> {
 #[doc(hidden)]
 unsafe impl<I> core::iter::InPlaceIterable for IntoIter<I> {}
 
+#[must_use = "iterators are lazy and do nothing unless consumed"]
 #[cfg(feature = "unstable")]
 #[derive(Clone, Debug)]
 pub struct IntoIterSorted<T, const D: usize> {
@@ -1703,6 +1799,10 @@ impl<T: Ord, const D: usize, const N: usize> From<[T; N]> for DaryHeap<T, D> {
     /// }
     /// ```
     fn from(arr: [T; N]) -> Self {
+        // With newer Rust versions `Self::from_iter(arr)` should be used, as
+        // using `IntoIter::new` is deprecated from 1.59.0. However, this would
+        // require a MSRV of 1.53.0, and both are equivalent behind the scenes.
+        #[allow(deprecated)]
         core::array::IntoIter::new(arr).collect()
     }
 }
